void wifi_init() {

  sta_init();

  while (WiFi.waitForConnectResult() != WL_CONNECTED) {

    sta_cnt();
    Serial.println("STA connection failed, retrying...");
  }

  if ((WiFi.waitForConnectResult() == WL_CONNECTED)) {

    Serial.println("");
    Serial.println("Connected as STA");
    Serial.print("STA IP address: ");
    Serial.println(WiFi.localIP());
  }
}

void sta_cnt() {

  WiFi.begin(ssid_sta, password_sta);
}

void sta_init() {

  Serial.println("");
  Serial.println("Initializing STA...");

  delay(100);
  WiFi.mode(WIFI_OFF);
  delay(100);
  WiFi.mode(WIFI_STA);
}

void server_init() {

  Serial.println("");
  Serial.println("Initializing HTTP server...");

  flow_server.on("/", handleRoot);
  flow_server.on("/login", handleLogin);
  flow_server.on("/dashboard", handleDashboard);
  flow_server.on("/logout", handleLogout);
  flow_server.on("/calcbill", handleCalcBill);
  flow_server.on("/total", handleTotal);
  flow_server.on("/bathroom", handleBathroom);
  flow_server.on("/von", handleVon);
  flow_server.on("/voff", handleVoff);
  flow_server.onNotFound(handleNotFound);

  flow_server.begin();
  flow_server.send(401);

  Serial.println("HTTP server started");
}

void handleVon() {

  valve_stat = true;
  flow_server.sendHeader("Location", "/calcbill");
  flow_server.send(302, "text/plain", "");
}

void handleVoff() {

  valve_stat = false;
  flow_server.sendHeader("Location", "/calcbill");
  flow_server.send(302, "text/plain", "");
}

void handleCalcBill() {

  String snd = "<!DOCTYPE html> <html>\n";
  snd += "<head><meta name=\"viewport\" content=\"width=device-width, initial-scale=1.0, user-scalable=no\">\n";
  snd += "<title>Water Management</title>\n";
  snd += "<style>html { font-family: Helvetica; display: inline-block; margin: 0px auto; text-align: center;}\n";
  snd += "body{margin-top: 50px;} h1 {color: #444444;margin: 50px auto 30px;} h3 {color: #888;margin-bottom: 50px;}\n";
  snd += ".button {display: block;width: 180px;background-color: #1abc9c;border: none;color: white;padding: 13px 30px;text-decoration: none;font-size: 25px;margin: 0px auto 35px;cursor: pointer;border-radius: 4px;}\n";
  snd += ".button-root {background-color: #4272f5;}\n";
  snd += ".button-root:active {background-color: #1e3166;}\n";
  snd += "p {font-size: 14px;color: #888;margin-bottom: 10px;}\n";
  snd += "</style>\n";
  snd += "</head>\n";
  snd += "<body>\n";
  snd += "<h1>Water Management</h1>\n";
  snd += "<h3>Bill Summery</h3>";
  snd += "<p><span style\"color: #ffffff\">Last Reading = 542</span></p>";
  snd += "<p><span style\"color: #ffffff\">Last month usage (units) = 34</span></p>";
  snd += "<p><span style\"color: #ffffff\">Cost per last month usage = Rs.1330.00</span></p>";
  snd += "<p>&nbsp;</p>\n";
  snd += "<p>&nbsp;</p>\n";
  snd += "<h3>Water Supply connectivity</h3>";

  if (valve_stat) {

    snd += "<p>Connection Status: Connected</p><a class=\"button button-root\" href=\"/voff\">Disconnect</a>\n";
  }
  else {

    snd += "<p>Connection Status: Disconnected</p><a class=\"button button-root\" href=\"/von\">Connect</a>\n";
  }
  snd += "<p><a href=\"/dashboard\">Dashboard</a></p>";
  snd += "<p><span style\"color: #ff0000\"><a style=\"color: #ff0000;\" href=\"/logout\">Logout</a></span></p>";
  snd += "</body>\n";
  snd += "</html>\n";
  flow_server.send(200, "text/html", snd);
}

void handleTotal() {

  String snd = "<!DOCTYPE html> <html>\n";
  snd += "<head><meta name=\"viewport\" content=\"width=device-width, initial-scale=1.0, user-scalable=no\">\n";
  snd += "<title>Water Management</title>\n";
  snd += "<style>html { font-family: Helvetica; display: inline-block; margin: 0px auto; text-align: center;}\n";
  snd += "body{margin-top: 50px;} h1 {color: #444444;margin: 50px auto 30px;} h3 {color: #888;margin-bottom: 50px;}\n";
  snd += ".button {display: block;width: 80px;background-color: #1abc9c;border: none;color: white;padding: 13px 30px;text-decoration: none;font-size: 25px;margin: 0px auto 35px;cursor: pointer;border-radius: 4px;}\n";
  snd += ".button-root {background-color: #4272f5;}\n";
  snd += ".button-root:active {background-color: #1e3166;}\n";
  snd += "p {font-size: 14px;color: #888;margin-bottom: 10px;}\n";
  snd += "</style>\n";
  snd += "</head>\n";
  snd += "<body>\n";
  snd += "<h1>Water Management</h1>\n";
  snd += "<h3>Total Consumption Summery</h3>";
  snd += "<p>&nbsp;</p>\n";
  snd += "<p>&nbsp;</p>\n";
  snd += "<iframe width=\"450\" height=\"260\" style=\"border: 1px solid #cccccc;\" src=\"https://thingspeak.com/channels/1153524/charts/1?bgcolor=%23ffffff&color=%23d62020&dynamic=true&results=10&title=Total+Water+Consumption&type=line&xaxis=Time&yaxis=Consumption+%28L%29\"></iframe>";
  snd += "<p>&nbsp;</p>\n";
  snd += "<p>&nbsp;</p>\n";
  snd += "<iframe width=\"450\" height=\"260\" style=\"border: 1px solid #cccccc;\" src=\"https://thingspeak.com/channels/1153524/charts/2?bgcolor=%23ffffff&color=%23d62020&dynamic=true&results=10&title=Total+Water+Flow+Rate&type=line&xaxis=Time&yaxis=Flow+Rate+%28L%2Fmin%29\"></iframe>";
  snd += "<p>&nbsp;</p>\n";
  snd += "<p><a href=\"/dashboard\">Dashboard</a></p>";
  snd += "<p><span style\"color: #ff0000\"><a style=\"color: #ff0000;\" href=\"/logout\">Logout</a></span></p>";
  snd += "</body>\n";
  snd += "</html>\n";
  flow_server.send(200, "text/html", snd);
}

void handleBathroom() {

  String snd = "<!DOCTYPE html> <html>\n";
  snd += "<head><meta name=\"viewport\" content=\"width=device-width, initial-scale=1.0, user-scalable=no\">\n";
  snd += "<title>Water Management</title>\n";
  snd += "<style>html { font-family: Helvetica; display: inline-block; margin: 0px auto; text-align: center;}\n";
  snd += "body{margin-top: 50px;} h1 {color: #444444;margin: 50px auto 30px;} h3 {color: #888;margin-bottom: 50px;}\n";
  snd += ".button {display: block;width: 80px;background-color: #1abc9c;border: none;color: white;padding: 13px 30px;text-decoration: none;font-size: 25px;margin: 0px auto 35px;cursor: pointer;border-radius: 4px;}\n";
  snd += ".button-root {background-color: #4272f5;}\n";
  snd += ".button-root:active {background-color: #1e3166;}\n";
  snd += "p {font-size: 14px;color: #888;margin-bottom: 10px;}\n";
  snd += "</style>\n";
  snd += "</head>\n";
  snd += "<body>\n";
  snd += "<h1>Water Management</h1>\n";
  snd += "<h3>Bathroom Consumption Summery</h3>";
  snd += "<p>&nbsp;</p>\n";
  snd += "<p>&nbsp;</p>\n";
  snd += "<iframe width=\"450\" height=\"260\" style=\"border: 1px solid #cccccc;\" src=\"https://thingspeak.com/channels/1153524/charts/3?bgcolor=%23ffffff&color=%23d62020&dynamic=true&results=10&title=Bathroom+Water+Flow+Rate&type=line&xaxis=Time&yaxis=Flow+Rate+%28L%2Fmin%29\"></iframe>";
  snd += "<p>&nbsp;</p>\n";
  snd += "<p>&nbsp;</p>\n";
  snd += "<<iframe width=\"450\" height=\"260\" style=\"border: 1px solid #cccccc;\" src=\"https://thingspeak.com/channels/1153524/charts/4?bgcolor=%23ffffff&color=%23d62020&dynamic=true&results=10&title=Bathroom+Water+Consumption&type=line&xaxis=Consumption+%28L%29&yaxis=Time\"></iframe>";
  snd += "<p>&nbsp;</p>\n";
  snd += "<p><a href=\"/dashboard\">Dashboard</a></p>";
  snd += "<p><span style\"color: #ff0000\"><a style=\"color: #ff0000;\" href=\"/logout\">Logout</a></span></p>";
  snd += "</body>\n";
  snd += "</html>\n";
  flow_server.send(200, "text/html", snd);
}

void handleLogout() {

  if (logged_out) {

    flow_server.sendHeader("Location", "/");
    flow_server.send(302, "text/plain", "");
    logged_out = false;
  }
  else {

    String snd = "<!DOCTYPE html> <html>\n";
    snd += "<head><meta name=\"viewport\" content=\"width=device-width, initial-scale=1.0, user-scalable=no\">\n";
    snd += "<title>Water Management</title>\n";
    snd += "<script>function refresh(refreshPeriod){setTimeout('location.reload(true)', refreshPeriod);}window.onload = refresh(2000);</script>";
    snd += "<style>html { font-family: Helvetica; display: inline-block; margin: 0px auto; text-align: center;}\n";
    snd += "body{margin-top: 50px;} h1 {color: #444444;margin: 50px auto 30px;} h3 {color: #888;margin-bottom: 50px;}\n";
    snd += "p {font-size: 14px;color: #888;margin-bottom: 10px;}\n";
    snd += "</style>\n";
    snd += "</head>\n";
    snd += "<body>\n";
    snd += "<h1>Water Management</h1>\n";

    snd += "<h3>You have been logged-out.</h3>\n";

    snd += "<p>Redirecting...</p>";
    snd += "</body>\n";
    snd += "</html>\n";

    flow_server.send(401, "text/html", snd);

    logged_out = true;
  }
}

void handleDashboard() {

  if (is_admin()) {

    flow_server.send(200, "text/html", get_snd_dash());
  }
  else {

    fail_auth();
  }
}

String get_snd_dash() {

  String snd = "<!DOCTYPE html> <html>\n";
  snd += "<head><meta name=\"viewport\" content=\"width=device-width, initial-scale=1.0, user-scalable=no\">\n";
  snd += "<title>Garden Remote</title>\n";
  snd += "<script>function refresh(refreshPeriod){setTimeout('location.reload(true)', refreshPeriod);}window.onload = refresh(" + String(web_page_refresh_interval) + ");</script>";
  snd += "<style>html { font-family: Helvetica; display: inline-block; margin: 0px auto; text-align: center;}\n";
  snd += "body{margin-top: 50px;} h1 {color: #444444;margin: 50px auto 30px;} h3 {color: #888;margin-bottom: 50px;}\n";
  snd += ".button {display: block;width: 160px;background-color: #1abc9c;border: none;color: white;padding: 13px 30px;text-decoration: none;font-size: 25px;margin: 0px auto 35px;cursor: pointer;border-radius: 4px;}\n";
  snd += ".button-bill {background-color: #4272f5;}\n";
  snd += ".button-bill:active {background-color: #1e3166;}\n";
  snd += ".button-total {background-color: #4272f5;}\n";
  snd += ".button-total:active {background-color: #1e3166;}\n";
  snd += ".button-bathroom {background-color: #4272f5;}\n";
  snd += ".button-bathroom:active {background-color: #1e3166;}\n";
  snd += "p {font-size: 14px;color: #888;margin-bottom: 10px;}\n";
  snd += "</style>\n";
  snd += "</head>\n";
  snd += "<body>\n";
  snd += "<h1>Water Management</h1>\n";
  snd += "<h3>Dashboard</h3>\n";

  NTPunixTime = NTPtime;
  setTime(NTPunixTime);

  snd += "<p><span style=\"color: #444444;\">Date:- " + String(day()) + "/" + String(month()) + "/" + String(year()) + "&nbsp; &nbsp; &nbsp; &nbsp; &nbsp;Time:- " + String(hour()) + ":" + String(minute()) + ":" + String(second()) + "</span></p>\n";

  snd += "<p>&nbsp;</p>\n";

  snd += "</p><a class=\"button button-bill\" href=\"/calcbill\">Calculate Bill</a>\n";
  snd += "</p><a class=\"button button-total\" href=\"/total\">Total Consumption</a>\n";
  snd += "</p><a class=\"button button-bathroom\" href=\"/bathroom\">Bathroom Consumption</a>\n";


  // snd += "<p><a href=\"/adminlogin\">System Settings</a></p>\n";
  //snd += "<p><a href=\"/update\">System Update</a></p>\n";
  snd += "<p><span style\"color: #ff0000\"><a style=\"color: #ff0000;\" href=\"/logout\">Logout</a></span></p>";
  snd += "</body>\n";
  snd += "</html>\n";

  return snd;
}

void handleRoot() {

  String snd = "<!DOCTYPE html> <html>\n";
  snd += "<head><meta name=\"viewport\" content=\"width=device-width, initial-scale=1.0, user-scalable=no\">\n";
  snd += "<title>Water Management</title>\n";
  snd += "<style>html { font-family: Helvetica; display: inline-block; margin: 0px auto; text-align: center;}\n";
  snd += "body{margin-top: 50px;} h1 {color: #444444;margin: 50px auto 30px;} h3 {color: #888;margin-bottom: 50px;}\n";
  snd += ".button {display: block;width: 80px;background-color: #1abc9c;border: none;color: white;padding: 13px 30px;text-decoration: none;font-size: 25px;margin: 0px auto 35px;cursor: pointer;border-radius: 4px;}\n";
  snd += ".button-root {background-color: #4272f5;}\n";
  snd += ".button-root:active {background-color: #1e3166;}\n";
  snd += "p {font-size: 14px;color: #888;margin-bottom: 10px;}\n";
  snd += "</style>\n";
  snd += "</head>\n";
  snd += "<body>\n";
  snd += "<h1>Water Management</h1>\n";
  snd += "<h3>Click <strong>LOGIN</strong> to continue.</h3>";
  snd += "<a class=\"button button-root\" href=\"/login\">LOGIN</a>\n";
  snd += "</body>\n";
  snd += "</html>\n";
  flow_server.send(200, "text/html", snd);
}

void handleNotFound() {

  String snd = "<!DOCTYPE html> <html>\n";
  snd += "<head><meta name=\"viewport\" content=\"width=device-width, initial-scale=1.0, user-scalable=no\">\n";
  snd += "<title>Garden Remote</title>\n";
  snd += "<style>html { font-family: Helvetica; display: inline-block; margin: 0px auto; text-align: center;}\n";
  snd += "body{margin-top: 50px;} h1 {color: #444444;margin: 50px auto 30px;} h3 {color: #ff0000;margin-bottom: 50px;}\n";
  snd += "p {font-size: 14px;color: #888;margin-bottom: 10px;}\n";
  snd += "</style>\n";
  snd += "</head>\n";
  snd += "<body>\n";
  snd += "<h1>Error 404!</h1>\n";
  snd += "<h3>...Destination not found...</h3>\n";
  snd += "<p><a href=\"/\">Back to Top</a></p>\n";
  snd += "</body>\n";
  snd += "</html>\n";

  flow_server.send(404, "text/html", snd);
}

void handleLogin() {

  if (is_admin()) {

    flow_server.sendHeader("Location", "/dashboard");
    flow_server.send(302, "text/plain", "");
  }
  else {

    fail_auth();
  }
}

bool is_admin() {

  if (flow_server.authenticate(admin_username, admin_password)) {

    return true;
  }
  else {

    return false;
  }
}

void fail_auth() {

  return flow_server.requestAuthentication(DIGEST_AUTH, www_realm, authFailResponse);
}
